﻿--[[---------------------------------------------------------------------------

Copyright (c) 2008 by K. Scott Piel 
All Rights Reserved

E-mail: < kscottpiel@gmail.com >
Web:    < http://www.scottpiel.com >

This file is part of nUI.

    nUI is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    nUI is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with nUI.  If not, see <http://www.gnu.org/licenses/>.
	
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE 
USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

--]]---------------------------------------------------------------------------

if not nUI then nUI = {}; end
if not nUI_Unit then nUI_Unit = {}; end
if not nUI_UnitOptions then nUI_UnitOptions = {}; end
if not nUI_DefaultConfig then nUI_DefaultConfig = {}; end
if not nUI_Profile then nUI_Profile = {}; end;

local GetComboPoints = GetComboPoints;

-- stolen directly from RuneFrame.lua for now -- need to nUIize this

local RUNETYPE_BLOOD = 1;
local RUNETYPE_UNHOLY = 2;
local RUNETYPE_FROST = 3;
local RUNETYPE_DEATH = 4;

local iconTextures = 
{
	[RUNETYPE_BLOOD]  = "Interface\\PlayerFrame\\UI-PlayerFrame-Deathknight-Blood",
	[RUNETYPE_UNHOLY] = "Interface\\PlayerFrame\\UI-PlayerFrame-Deathknight-Unholy",
	[RUNETYPE_FROST]  = "Interface\\PlayerFrame\\UI-PlayerFrame-Deathknight-Frost",
	[RUNETYPE_DEATH]  = "Interface\\PlayerFrame\\UI-PlayerFrame-Deathknight-Death",
};

local runeColors = 
{
	[RUNETYPE_BLOOD]  = {1, 0, 0},
	[RUNETYPE_UNHOLY] = {0, 0.5, 0},
	[RUNETYPE_FROST]  = {0, 1, 1},
	[RUNETYPE_DEATH]  = {0.8, 0.1, 1},
};

-- rune ordering: 1-2 are blood, 3-4 are unholy, 5-6 are frost

local runeOrder = { 1, 2, 5, 6, 3, 4 };

-------------------------------------------------------------------------------
-- unit runes event management

if not nUI_Unit.Drivers then 
	nUI_Unit.Drivers = CreateFrame( "Frame", "nUI_UnitDrivers", WorldFrame ); 
end

nUI_Profile.nUI_UnitRunes       = {};
nUI_Profile.nUI_UnitRunes.Frame = {};

local ProfileCounter      = nUI_Profile.nUI_UnitRunes;
local FrameProfileCounter = nUI_Profile.nUI_UnitRunes.Frame;

local frame = CreateFrame( "Frame", "$parent_Runes", nUI_Unit.Drivers )

local RunesCallbacks    = {};
local RunesUnits        = {};
local RunesStatus       = {};

nUI_Unit.Drivers.Runes  = frame;

-------------------------------------------------------------------------------

local function onRunesEvent()

	nUI_ProfileStart( ProfileCounter, "onRunesEvent", event );		
	frame.newUnitInfo( "player", nUI_Unit.PlayerInfo );
	nUI_ProfileStop();
		
end

frame:SetScript( "OnEvent", onRunesEvent );
frame:RegisterEvent("RUNE_POWER_UPDATE");
frame:RegisterEvent("RUNE_TYPE_UPDATE");
frame:RegisterEvent("PLAYER_ENTERING_WORLD");

-------------------------------------------------------------------------------
-- this callback method is called when one of the unit IDs we are monitoring
-- for unit runes changes GUID

frame.newUnitInfo = function( list_unit, unit_info )

	nUI_ProfileStart( ProfileCounter, "newUnitInfo" );	
	
	local new_data  = nUI_Unit:updateRunesInfo( list_unit, unit_info );
	local callbacks = RunesCallbacks;
	local unitlist  = RunesUnits;
	
	nUI_Unit:notifyCallbacks( nUI_L["unit runes"], callbacks, unitlist, unit_info, list_unit, new_data );
	
	nUI_ProfileStop();
	
end

-------------------------------------------------------------------------------
-- add and remove callbacks from the list of unit runes listeners we manage
--
-- calling this method will return the current unit_info structure for this 
-- unit if it exists or nil if the unit does not exist at this time
--
-- Note: these callbacks will be notified both when the underlying GUID for the
--		 unit changes or when the runes info of the underlying GUID to the
--		 player changes. If the underlying unit does not exist, the callback
--		 will be passed a nil unit_info structure

function nUI_Unit:registerRunesCallback( unit_id, callback )
	
	nUI_ProfileStart( ProfileCounter, "registerRunesCallback" );	
	
	local unit_info = nil;
	
	if unit_id and callback then
		
		-- get the list of callbacks for this unit id and add this callback
		
		local list = RunesCallbacks[unit_id] or {};
		
		nUI:TableInsertByValue( list, callback );
		
		-- if this is a new unit id, add it to the callback list
		
		if not RunesCallbacks[unit_id] then
			RunesCallbacks[unit_id] = list;
		end
		
		-- if this is the first callback for the unit id, then register our
		-- event driver to receive notice when the GUID changes on this id
		
		if #list == 1 then
			nUI_Unit:registerUnitChangeCallback( unit_id, nUI_Unit.Drivers.Runes );
		end
		
		-- collect runes information for this unit as we know it at this time
	
		unit_info = nUI_Unit:getUnitInfo( unit_id );
		
		if unit_info then
			nUI_Unit:updateRunesInfo( unit_id, unit_info );
		end
	end
	
	nUI_ProfileStop();
	
	return unit_info;
	
end

function nUI_Unit:unregisterRunesCallback( unit_id, callback )
	
	nUI_ProfileStart( ProfileCounter, "unregisterRunesCallback" );	
	
	if unit_id and callback then
		
		-- get the list of current callbacks for this unit ud and remove this callback
		
		local list = RunesCallbacks[unit_id] or {};
		
		nUI:TableRemoveByValue( list, callback );
		
		-- if that's the last callback in the list, then remove our event handler of
		-- the list of unit change callbacks for that unit it
		
		if #list == 0 then
			nUI_Unit:unregisterUnitChangeCallback( unit_id, nUI_Unit.Drivers.Runes );
		end
	end

	nUI_ProfileStop();
	
end

-------------------------------------------------------------------------------
-- update the runes information for this unit
--
-- note: it is the caller's responsibility to ensure that the unit_info being
--       passed belongs to the unit_id that is passed. Generally third party
--       consumers of unit_info should not call this method, rather they 
--       should use the callback registration system to get change notices
--       and let the nUI unit driver engine do the updating. If you MUST call
--       this method, you should first test that the following condition 
--       evaluates as true: UnitGUID( unit_id ) == unit_info.guid
--
-- returns the updated unit information structure for the current GUID
-- if the data has changed, otherwise returns nil if nothing changed

function nUI_Unit:updateRunesInfo( unit_id, unit_info )

	nUI_ProfileStart( ProfileCounter, "updateRunesInfo" );	
	
	local modified  = false;
	
	if unit_info then

		local has_runes = unit_info.class == "DEATHKNIGHT" and UnitIsUnit( unit_id, "player" );		
		local runes     = has_runes and unit_info.runes or nil;
		
		if has_runes then
				
			-- player is entering the world
					
			runes = {};
			
			for i=1,6 do
				
				local start, duration, ready = GetRuneCooldown( i );
				
				runes[i] =
				{
					type     = GetRuneType( i ),
					start    = start,
					duration = duration,
					ready    = ready,
					noflash  = true,
				};
			end				
		end
		
		-- reset the local globals so they are not reused in the next iteration
		
		rune   = nil;
		usable = nil;
		
		-- if we are adding or removing runes from the unit info then
		-- that's all we need to worry about
		
		if (unit_info.runes and not runes)
		or (not unit_info.runes and runes)
		then
			
			modified               = true;
			unit_info.modified     = true;
			unit_info.last_change  = GetTime();
			unit_info.runes        = runes;
			
		-- otherwise we need to loop through the runes if there are
		-- any and check each one for changes
		
		elseif runes then
			
			for i=1,#runes do
				
				local oldRune = unit_info.runes[i];
				local newRune = runes[i];
				
				if oldRune.type     ~= newRune.type
				or oldRune.ready    ~= newRune.ready
				or oldRune.duration ~= newRune.duration
				or oldRune.start    ~= newRune.start
				then
					
					modified               = true;
					unit_info.modified     = true;
					unit_info.last_change  = GetTime();
					unit_info.runes[i]     = runes[i];
					
				end
			end
		end
	end
	
	nUI_ProfileStop();
	
	return modified and unit_info or nil;
	
end

-------------------------------------------------------------------------------
-- update all of the registered unit runes listeners, even if there's no 
-- change in data... typically used when entering the world

function nUI_Unit:refreshRunesCallbacks()

	nUI_ProfileStart( ProfileCounter, "refreshRunesCallbacks" );	
	
	nUI_Unit:refreshCallbacks( 
	
		nUI_L["unit runes"], RunesCallbacks, RunesUnits, 
	
		function( list_unit, unit_info ) 
			nUI_Unit:updateRunesInfo( list_unit, unit_info ); 
		end 
	);

	nUI_ProfileStop();
	
end

-------------------------------------------------------------------------------
-- create a new unit runes frame

function nUI_Unit:createRunesFrame( parent, unit_id, id, options )

	nUI_ProfileStart( ProfileCounter, "createRunesFrame" );	
	
	local frame = nUI_Unit:createFrame( "$parent_Runes"..(id or ""), parent, unit_id, false );

	frame.runes = {};
	frame.Super = {};

	for i=1,6 do
	
		local rune = CreateFrame( "Button", "$parent_Rune"..i, frame, "RuneButtonIndividualTemplate" );		
		local name = rune:GetName();
		
		rune.rune     = _G[name.."Rune"];
		rune.fill     = _G[name.."Fill"];
		rune.shine    = _G[name.."ShineTexture"];
		rune.cooldown = _G[name.."Cooldown"];

		rune.rune:Show();
		rune:EnableMouse( false );
		rune:SetScript( "OnLoad", nil );
		
		frame.runes[i] = rune;
		
	end

	-- called when the underlying GUID for the unit changes or when the
	-- content of the GUID is updated

	frame.Super.newUnitInfo = frame.newUnitInfo;
	frame.newUnitInfo       = function( list_unit, unit_info )
		
		nUI_ProfileStart( FrameProfileCounter, "newUnitInfo" );	
	
		frame.Super.newUnitInfo( list_unit, unit_info );
		
		if frame.enabled then
			nUI_Unit:updateRunesFrame( frame );
		end
		
		nUI_ProfileStop();
		
	end
	
	-- setting enabled to false will prevent the frame from updating when new
	-- unit information is received (saves framerate). Setting enabled true will
	-- call the frame to immediately update if its content has changed since it
	-- was disabled

	frame.Super.setEnabled = frame.setEnabled;	
	frame.setEnabled       = function( enabled )
		
		nUI_ProfileStart( FrameProfileCounter, "setEnabled" );	
	
		local prior_state = frame.enabled;
		
		frame.Super.setEnabled( enabled );
		
		if frame.enabled ~= prior_state then
		
			if frame.enabled then
				frame.unit_info = nUI_Unit:registerRunesCallback( frame.unit, frame );
				nUI_Unit:updateRunesFrame( frame );
			else
				nUI_Unit:unregisterRunesCallback( frame.unit, frame );
			end
		end

		nUI_ProfileStop();
		
	end
	
	-- used to change the scale of the frame... rather than the Bliz widget frame:SetScale()
	-- this method actually recalculates the size of the frame and uses frame:SetHeight()
	-- and frame:SetWidth() to reflect the actual size of the frame.
	
	frame.Super.applyScale = frame.applyScale;	
	frame.applyScale       = function( scale )

		nUI_ProfileStart( FrameProfileCounter, "applyScale" );	
	
		frame.Super.applyScale( scale );
		
		local hgap = frame.options and frame.options.hgap or 0;
		local last_texture, first, left, right;
				
		hgap = hgap * frame.scale * nUI.scale;
		
		if frame.runes[1].size   ~= frame.size 
		or frame.runes[1].width  ~= frame.width
		or frame.runes[1].height ~= frame.height 
		or frame.runes[1].inset  ~= frame.inset
		or frame.runes[1].hgap   ~= hgap
		then
			
			frame.runes[1].size   = frame.size;
			frame.runes[1].width  = frame.width;
			frame.runes[1].height = frame.height;
			frame.runes[1].inset  = frame.inset;
			frame.runes[1].hgap   = hgap;
			
			for i=1, #frame.runes do
				
				local texture = frame.runes[i];
				
				texture:ClearAllPoints();
				
				if options.orient == "RIGHT" then
					if i == 1 then texture:SetPoint( "RIGHT", frame, "RIGHT", 0, 0 );
					else texture:SetPoint( "RIGHT", last_texture, "LEFT", -hgap, 0 );
					end
				elseif options.orient == "CENTER" then
					if i == 1 then texture:SetPoint( "LEFT", frame, "CENTER", hgap/2, 0 ); first = texture;
					elseif (i % 2) == 0 then texture:SetPoint( "RIGHT", left or first, "LEFT", -hgap, 0 ); left = texture;
					else texture:SetPoint( "LEFT", right or first, "RIGHT", hgap, 0 ); right = texture;
					end
				elseif options.orient == "TOP" then
					if i == 1 then texture:SetPoint( "TOP", frame, "TOP", 0, 0 );
					else texture:SetPoint( "TOP", last_texture, "BOTTOM", 0, -hgap );
					end
				elseif options.orient == "BOTTOM" then
					if i == 1 then texture:SetPoint( "BOTTOM", frame, "BOTTOM", 0, 0 );
					else texture:SetPoint( "BOTTOM", last_texture, "TOP", 0, hgap );
					end
				elseif options.orient == "MIDDLE" then
					if i == 1 then texture:SetPoint( "BOTTOM", frame, "CENTER", 0, hgap/2 ); first = texture;
					elseif (i % 2) == 0 then texture:SetPoint( "TOP", left or first, "BOTTOM", 0, -hgap ); left = texture;
					else texture:SetPoint( "BOTTOM", right or last_first, "TOP", 0, hgap ); right = texture;
					end
				else
					if i == 1 then texture:SetPoint( "LEFT", frame, "LEFT", 0, 0 );
					else texture:SetPoint( "LEFT", last_texture, "RIGHT", hgap, 0 );
					end				
				end
				
				last_texture = texture;
				
				frame.runes[i]:SetScale( ((frame.size or frame.width) - frame.inset) / frame.runes[i]:GetWidth() );

			end
		end		

		nUI_ProfileStop();
		
	end
	
	-- applies the set of frame options to this frame. Typically called when the frame 
	-- is first created or when the user changes options via config.
	
	frame.Super.applyOptions = frame.applyOptions;
	frame.applyOptions       = function( options )

		nUI_ProfileStart( FrameProfileCounter, "applyOptions" );	
	
		frame.Super.applyOptions( options );		
		
		for i=1,6 do
			frame.runes[i]:SetID( options.order and options.order[i] or runeOrder[i] );
		end
			
		frame.applyScale( options.scale or frame.scale or 1 );
				
		nUI_Unit:updateRunesFrame( frame );
		
		nUI_ProfileStop();
		
	end

	-- initiate the frame
	
	frame.unit_info = nUI_Unit:registerRunesCallback( frame.unit, frame );
	
	frame.applyOptions( options );
	
	nUI_ProfileStop();
	
	return frame;
	
end

-------------------------------------------------------------------------------
-- remove a unit runes frame

function nUI_Unit:deleteRunesFrame( frame )

	nUI_ProfileStart( ProfileCounter, "deleteRunesFrame" );

	nUI_Unit:unregisterRunesCallback( frame.unit, frame );
	nUI_Unit:deleteFrame( frame );
	
	nUI_ProfileStop();
	
end

-------------------------------------------------------------------------------
-- display the appropriate icon for the unit's runes status
--
-- note: this method expends extra energy in state management... as in knowing
--       exactly what state it is currently in and only updating the frame text,
--       content, colors, alphas, etc. when a change in state occurs. The extra
--       effort is spent on state management in order to reduce impact to the
--       graphis engine so as to preserve frame rate. It costs far less to check
--		 a memory value that and burn through the graphics geometry. It does not
--       matter how many times the unit changes GUID or how many times this 
--       method will call, it will only alter the graphics elements when its
--       relative state changes.

function nUI_Unit:updateRunesFrame( frame )
	
	nUI_ProfileStart( ProfileCounter, "updateRunesFrame" );

	local unit_info = frame.unit_info;
	
	-- if there is no unit or we don't know it's runes status, then hide the icons
	
	if not unit_info 
	or not unit_info.runes
	then
		
		if frame.active then
			frame.active = false;
			frame:SetAlpha( 0 );
		end
	
	-- otherwise, show the icon
	
	else

		-- if the runes icon is hidden, show it
		
		if not frame.active then
			frame.active = true;
			frame:SetAlpha( 1 );
		end
		
		for i=1,#unit_info.runes do
			
			local button = frame.runes[i];
			local rune   = unit_info.runes[button:GetID()];

			if not button.rune:IsShown() 
			and not InCombatLockdown()
			then button.rune:Show();
			end
			
			if not rune.usable then
				CooldownFrame_SetTimer( button.cooldown, rune.start, rune.duration, rune.ready and 0 or 1 );
			end
			
			if rune.type
			and rune.type ~= button.rune.type 
			then
			
				button.rune.type = type;
				
				if not rune.noflash then
					button.shine:SetVertexColor( unpack( runeColors[rune.type] ) );
					RuneButton_ShineFadeIn( button.shine );
				end
			
				button.rune:SetTexture( iconTextures[rune.type] );
				button.rune:SetAlpha( 1 );
				button.tooltipText = _G["COMBAT_TEXT_RUNE_"..runeMapping[rune.type]];
				
			elseif not rune.type then
				
				button.rune:SetAlpha( 0 );
				button.tooltipText = nil;

			end			
		end
	end
	
	nUI_ProfileStop();
	
end
